/*
 * Copyright 2002 Sun Microsystems, Inc. All Rights Reserved.
 */

package javax.mail;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import java.util.Enumeration;
import java.util.Vector;

/**
 * Security related methods that only work on J2SE 1.2 and newer.
 */
final class SecuritySupport12 extends SecuritySupport {

	public ClassLoader getContextClassLoader() {
		return (ClassLoader) AccessController.doPrivileged(new PrivilegedAction() {
			public Object run() {
				ClassLoader cl = null;
				try {
					cl = Thread.currentThread().getContextClassLoader();
				} catch (SecurityException ex) {}
				return cl;
			}
		});
	}

	public InputStream getResourceAsStream(final Class c, final String name) throws IOException {
		try {
			return (InputStream) AccessController.doPrivileged(new PrivilegedExceptionAction() {
				public Object run() throws IOException {
					return c.getResourceAsStream(name);
				}
			});
		} catch (PrivilegedActionException e) {
			throw (IOException)e.getException();
		}
	}

	public URL[] getResources(final ClassLoader cl, final String name) {
		return (URL[]) AccessController.doPrivileged(new PrivilegedAction() {
			public Object run() {
				URL[] ret = null;
				try {
					Vector v = new Vector();
					Enumeration e = cl.getResources(name);
					while (e != null && e.hasMoreElements()) {
						URL url = (URL) e.nextElement();
						if (url != null)
							v.addElement(url);
					}
					if (v.size() > 0) {
						ret = new URL[v.size()];
						v.copyInto(ret);
					}
				} catch (IOException ioex) {
				} catch (SecurityException ex) {}
				return ret;
			}
		});
	}

	public URL[] getSystemResources(final String name) {
		return (URL[]) AccessController.doPrivileged(new PrivilegedAction() {
			public Object run() {
				URL[] ret = null;
				try {
					Vector v = new Vector();
					Enumeration e = ClassLoader.getSystemResources(name);
					while (e != null && e.hasMoreElements()) {
						URL url = (URL) e.nextElement();
						if (url != null)
							v.addElement(url);
					}
					if (v.size() > 0) {
						ret = new URL[v.size()];
						v.copyInto(ret);
					}
				} catch (IOException ioex) {
				} catch (SecurityException ex) {}
				return ret;
			}
		});
	}

	public InputStream openStream(final URL url) throws IOException {
		try {
			return (InputStream) AccessController.doPrivileged(new PrivilegedExceptionAction() {
				public Object run() throws IOException {
					return url.openStream();
				}
			});
		} catch (PrivilegedActionException e) {
			throw (IOException)e.getException();
		}
	}

}
